#include <stdio.h>
#include "NoFLA_Apply_td_LU_var30a.h"
#include "NoFLA_Apply_td_ipiv_var1.h"

#define min( a, b ) ( (a) < (b) ? (a) : (b) )


/* ========================================================================= */
void NoFLA_Apply_td_LU_var30a( int nb_alg, int m_G, int n_G, int n_D,
        double * buff_D, int ldim_D,
        double * buff_S, int ldim_S,
        int * buff_P,
        double * buff_F, int ldim_F,
        double * buff_G, int ldim_G ) {
/* Apply several block gaussian transformations defined by (Di,Pi,Si) to
   matrix [ F; G ] from left side.
     for i = 1, n_D/nb_alg
       [ F; G ] := apply_td_lu( [ I; Di ], Pi, Si, [ F; G ] ).
   Arguments:
     nb_alg                        Block size to be used in the factorization.
     D       in      m_G x n_D     Bottom part of matrix with gaussian trans.
                                   Defined by ( buff_D, ldim_D ).
     S       in      nb_alg x n_D  Matrix with triangular factors S.
                                   Defined by ( buff_S, ldim_S ).
     P       in      n_D           Vector with pivots.
                                   Defined by ( buff_P ).
     F       in/out  n_D x n_G     Top part of matrix to be updated.
                                   Defined by ( buff_F, ldim_F ).
     G       in/out  m_G x n_G     Bottom part of matrix to be updated.
                                   Defined by ( buff_G, ldim_G ).
*/
  double   * buff_S1, * buff_D1, * buff_F1;
  int      * buff_P1;
  double   d_one = 1.0, d_m_one = -1.0;
  int      b, k;

  /* Main loop. */
  for( k = 0; k < n_D; k += nb_alg ) {
    b = min( nb_alg, n_D - k );

    /* Some initializations. */
    buff_D1 = &( buff_D[ ldim_D * k + 0 ] );  // D( 0, k )
    buff_F1 = &( buff_F[ ldim_F * 0 + k ] );  // F( k, 0 )
    buff_S1 = &( buff_S[ ldim_S * k + 0 ] );  // S( 0, k )
    buff_P1 = &( buff_P[ k ] );               // P( k )

    /* Apply pivoting from P1 to [ F; G ]. */
    //// MyFLA_Apply_td_ipiv_var1( P1, F1, G, m_F, m_F0=k );
    NoFLA_Apply_td_ipiv_var1( b, n_G, m_G,
        buff_P1, buff_F1, ldim_F, buff_G, ldim_G, n_D, k );

    /* F1 := inv( S1 ) * F1. */
    //// FLA_Trsm( FLA_LEFT, FLA_LOWER_TRIANGULAR, 
    ////           FLA_NO_TRANSPOSE, FLA_UNIT_DIAG,
    ////           FLA_ONE, S1, F1 );
    dtrsm_( "Left", "Lower", "No transpose", "Unit", 
            & b, & n_G, & d_one, 
            buff_S1, & ldim_S, buff_F1, & ldim_F );

    /* G := G - D1 * F1. */
    //// FLA_Gemm( FLA_NO_TRANSPOSE, FLA_NO_TRANSPOSE, 
    ////           FLA_MINUS_ONE, D1, F1, FLA_ONE, G );
    dgemm_( "No transpose", "No transpose", 
            & m_G, & n_G, & b,
            & d_m_one, buff_D1, & ldim_D, buff_F1, & ldim_F,
            & d_one, buff_G, & ldim_G );
  }
}

