#include "NoFLA_Apply_td_ipiv_var1.h"


/* Compute:  [ U; D ] := P * [ U; D ], where P is defined in vector ipiv. 
   Arguments:
     P  in      m_U        vector with pivots
     U  in/out  m_U x n_U  triangular matrix
     D  in/out  m_D x n_U  dense matrix
*/


int NoFLA_Apply_td_ipiv_var1( int m_U, int n_U, int m_D,
        int * buff_ipiv, 
        double * buff_U, int ldim_U, 
        double * buff_D, int ldim_D, 
        int m_all_U, int m_prev_U ) {

	if( ( m_U == 0 )||( n_U == 0 )||( m_D == 0 ) ) {
		return 0;
	}

	int i;
	for( i = 0; i < m_U; i++ ) {
		int i_max_pos = buff_ipiv[ i ];
		if( i_max_pos != ( m_prev_U + i + 1 ) ) {
			/* Do pivoting. */
			int i_D = i_max_pos - m_all_U - 1;
		      /* Swap row i-th of U and row i_max_pos-th of D. */
      			dswap_( & n_U, & buff_U[ i   ], & ldim_U, & buff_D[ i_D ], & ldim_D );
		} else {
		/* Do not pivoting. */
    		}
  	}
	return 0;
}

