#include <stdio.h>
#include "NoFLA_Compute_td_LU_var30a.h"
#include "NoFLA_LU_td_unb_var1.h"
#include "NoFLA_Apply_td_ipiv_var1.h"
#include "NoFLA_Utils.h"


#define min( a, b ) ( (a) < (b) ? (a) : (b) )


/* Compute LUpiv factorization of [ U; D ], where U is upper triangular, and
   returns pivots into vector ipiv, and triangular factors into matrix S.
   Arguments:
     nb_alg  in                    Block size to be used in the factorization.
     U       in/out  m_U x n_U     Top part of matrix to be factorized.
                                   Upper triang. Defined by ( buff_U, ldim_U ).
     D       in/out  m_D x n_U     Bottom part of matrix to be factorized.
                                   Dense. Defined by ( buff_D, ldim_D ).
     ipiv    out     min(m_U,n_U)  Vector with pivots.
                                   Defined by ( buff_ipiv ).
     S       out     nb_alg x n_U  Matrix with triangular factors S.
                                   Defined by ( buff_S, ldim_S ).
*/
void NoFLA_Compute_td_LU_var30a( int nb_alg, int m_U, int n_U, int m_D,
        double * buff_U, int ldim_U,
        double * buff_D, int ldim_D,
        double * buff_S, int ldim_S,
        int * buff_ipiv ) 
{
	double   * buff_S1, * buff_U11, * buff_U12, * buff_D1, * buff_D2;
	int      * buff_P1;
	int      n_U12, n_D2;
	double   d_one = 1.0, d_m_one = -1.0;
	int      b, k, mn_U;

	mn_U = min( m_U, n_U );

	for( k = 0; k < mn_U; k += nb_alg ) { 
    		b = min( nb_alg, mn_U - k );

    		n_U12 = n_U - k - b;
    		n_D2  = n_U - k - b;

    		buff_S1  = &( buff_S[ ldim_S * k + 0 ] );          // S( 0, k )
    		buff_U11 = &( buff_U[ ldim_U * k + k ] );          // U( k, k ) 
    		buff_U12 = &( buff_U[ ldim_U * ( k + b ) + k ] );  // U( k, k+b )
    		buff_D1  = &( buff_D[ ldim_D * k + 0 ] );          // D( 0, k )
    		buff_D2  = &( buff_D[ ldim_D * ( k + b ) + 0 ] );  // D( 0, k+b )
    		buff_P1  = &( buff_ipiv[ k ] );                    // ipiv( k )
																                                                               
    		// Copy U11 into S1
    		NoFLA_Copy_triu( b, b, buff_U11, ldim_U, buff_S1, ldim_S );
    		NoFLA_Zero_strict_lower_part( b, b, buff_S1, ldim_S );

    		// Factorize current block [ S1; D1 ] with pivoting in P1
    		NoFLA_LU_td_unb_var1( b, b, m_D,
        	buff_S1, ldim_S, buff_D1, ldim_D, buff_P1, m_U, k );

    		// Restore S1 into U11
    		NoFLA_Copy_triu( b, b, buff_S1, ldim_S, buff_U11, ldim_U );

    		// Apply pivoting from P1 to block: [ U12; D2  ]
    		NoFLA_Apply_td_ipiv_var1( b, n_U12, m_D, 
        	buff_P1, buff_U12, ldim_U, buff_D2, ldim_D, m_U, k );

    		// Update rest of matrix: [ U12; D2 ] with previous transformations
    		dtrsm_( "Left", "Lower", "No transpose", "Unit", 
            		& b, & n_U12, & d_one, 
            		buff_S1, & ldim_S, buff_U12, & ldim_U );

    		dgemm_( "No transpose", "No transpose", & m_D, & n_D2, & b, 
            		& d_m_one, buff_D1, & ldim_D, buff_U12, & ldim_U,
            		& d_one, buff_D2, & ldim_D );
	}
}

