#include "NoFLA_LU_td_unb_var1.h"
#include <stdio.h>

#define dabs(x)  ( (x) < 0.0 ? -(x) : (x) )
#define min( a, b ) ( (a) < (b) ? (a) : (b) )

extern int idamax();


/* ========================================================================= */
int NoFLA_LU_td_unb_var1( int m_U, int n_U, int m_D,
        double * buff_U, int ldim_U,
        double * buff_D, int ldim_D,
        int * buff_ipiv,
        int m_all_U, int m_prev_U ) {
/* Compute LU factorization of [ U; D ], where:
   Arguments:
     U     in/out  m_U x n_U     upper triangular matrix
     D     in/out  m_D x n_U     dense matrix
     ipiv  out     min(m_U,n_U)  vector with pivots
*/
  double  * buff_u12t, * buff_d1, * buff_D2;
  double  val_diag, val_max, inv_upsilon11, d_minus_one = -1.0;
  int     mn_U, i_max_pos, i, n_D2, i_one = 1;

  /* Some initializations. */
  mn_U = min( m_U, n_U );

  /* Main loop. */
  for( i = 0; i < mn_U; i++ ) {
    /* Some initializations. */
    buff_d1   = &( buff_D[ ldim_D * i + 0 ] );          // D( 0, i )
    buff_D2   = &( buff_D[ ldim_D * ( i + 1 ) + 0 ] );  // D( i, i+1 )
    buff_u12t = &( buff_U[ ldim_U * ( i + 1 ) + i ] );  // U( i, i+1 )
    n_D2      = n_U - i - 1;

    /* Compute position of maximum. */
    //// FLA_Iamax( d1, max_pos );
    i_max_pos = idamax_( & m_D, buff_d1, & i_one ) - 1;

    val_diag = dabs( buff_U[ ldim_U * i + i ] );
    val_max  = dabs( buff_D[ ldim_D * i + i_max_pos ] );
   
    if( val_max > val_diag ) {
      /* Do pivoting: Swap row i-th of U and row i_max_pos-th of D. */
      dswap_( & n_U, & buff_U[ i   ],       & ldim_U,
                     & buff_D[ i_max_pos ], & ldim_D );
      buff_ipiv[ i ] = m_all_U + i_max_pos + 1;
    } else {
      /* Do not pivoting. */
      buff_ipiv[ i ] = m_prev_U + i + 1;
    }

    /* Compute:  d1 = d1 / upsilon11. */
    //// FLA_Inv_scal( upsilon11, d1 );
    inv_upsilon11 = buff_U[ ldim_U * i + i ];
    if( inv_upsilon11 != 0.0 ) {
      inv_upsilon11 = 1.0 / inv_upsilon11;
    } else {
      /* Process this error. */
      //printf( "Found pivot zero %22.16g\n", inv_upsilon11 );
    }
    dscal_( & m_D, & inv_upsilon11, buff_d1, & i_one );

    /* Update rest of matrix of D:  D2 := D2 - d1 * u12t'. */
    //// FLA_Ger( FLA_MINUS_ONE, d1, u12t, D2 );
    dger_( & m_D, & n_D2, & d_minus_one, 
                  buff_d1, & i_one, buff_u12t, & ldim_U, 
                  buff_D2, & ldim_D );
  }

  return 0;
}

