#include "seq_kernels.h"
#include "NoFLA_LU_td_unb_var1.h"
#include "NoFLA_Apply_td_ipiv_var1.h"
#include "NoFLA_Utils.h"


#include <stdio.h>

#define min( a, b ) ( (a) < (b) ? (a) : (b) )


/* Compute LUpiv factorization of [ U; D ], where U is upper triangular, and
   returns pivots into vector ipiv, and triangular factors into matrix S.
   Arguments:
     nb_alg  in                    Block size to be used in the factorization.
     U       in/out  m_U x n_U     Top part of matrix to be factorized.
                                   Upper triang. Defined by ( buff_U, ldim_U ).
     D       in/out  m_D x n_U     Bottom part of matrix to be factorized.
                                   Dense. Defined by ( buff_D, ldim_D ).
     ipiv    out     min(m_U,n_U)  Vector with pivots.
                                   Defined by ( buff_ipiv ).
     S       out     nb_alg x n_U  Matrix with triangular factors S.
                                   Defined by ( buff_S, ldim_S ).
*/
void dgetrf_split( int t, int m, int n, int skip, double * buff_U, int ldim, double * buff_D, double * buff_S, int * buff_ipiv ) {
	double   d_one = 1.0, d_m_one = -1.0;
	// Rectangular blocks must be tall and skinny
	//int mn_U = min( m, n );
	int mn_U = n;

	buff_U += skip;
	int m_U = m - skip;

	int k;
	for( k = 0; k < mn_U; k += t ) { 
		int b = min( t, mn_U - k );

		int n_U12 = n - k - b;
    		//int n_D2  = n - k - b;

    		double *buff_S1  = &( buff_S[ t * k + 0 ] );          // S( 0, k )
    		double *buff_U11 = &( buff_U[ ldim * k + k ] );          // U( k, k ) 
    		double *buff_U12 = &( buff_U[ ldim * ( k + b ) + k ] );  // U( k, k+b )
    		double *buff_D1  = &( buff_D[ ldim * k + 0 ] );          // D( 0, k )
    		double *buff_D2  = &( buff_D[ ldim * ( k + b ) + 0 ] );  // D( 0, k+b )
    		int *buff_P1  = &( buff_ipiv[ k ] );                    // ipiv( k )

		// Copy U11 into S1
		NoFLA_Copy_triu( b, b, buff_U11, ldim, buff_S1, t );
		NoFLA_Zero_strict_lower_part( b, b, buff_S1, t );

		// Factorize current block [ S1; D1 ] with pivoting in P1
		NoFLA_LU_td_unb_var1( b, b, m, buff_S1, t, buff_D1, ldim, buff_P1, m_U, k );

		// Restore S1 into U11
		NoFLA_Copy_triu( b, b, buff_S1, t, buff_U11, ldim );


		// Apply pivoting from P1 to block: [ U12; D2  ]
		NoFLA_Apply_td_ipiv_var1( b, n_U12, m, buff_P1, buff_U12, ldim, buff_D2, ldim, m_U, k );

		// Update rest of matrix: [ U12; D2 ] with previous transformations
		dtrsm_( "Left", "Lower", "No transpose", "Unit", &b, &n_U12, &d_one, buff_S1, &t, buff_U12, &ldim );

    		dgemm_( "No transpose", "No transpose", &m, &n_U12, &b, &d_m_one, buff_D1, &ldim, buff_U12, &ldim, &d_one, buff_D2, &ldim );
	}
}
