#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>

#include "lupd_kernels.h"

#include "lu_config.h"
#include "lu_setup.h"
#include "lupd_check.h"
#include "lupd_utils.h"
#include "lupd_main.h"
#include "lupdh_main.h"
#include "lunat_main.h"
#include "lupdll_main.h"


int m;
int n;
int br;
int bc;
int t;
int h;
int reps;
int check;
int diagl;
int nt; 
int mt;


int main(int argc, char* argv[]) {
	if ( lu_config(argc,argv) ) {
		return 1;
	}

	int nr = nt * bc;
	int mr = mt * br;
	int ntleft = nr - n;  
	int mtleft = mr - m; 

	printf("%i it of input %ix%i -> %ix%i block %ix%i(%i) diag %i h %i %i\n", reps, m, n, mr, nr, br, bc, t, diagl, h);

  	double **Ah; 
  	double **Sh; 
  	int **Ph; 
	double *Aorig=NULL;
	if ( lu_setup(check, m, mr, n, nr, br, bc, t, h, mt, nt, mtleft, ntleft, &Ah, &Sh, &Ph, &Aorig) ) {
		return 2;
	}


	octave_matrix_def("A", m, mr, n, nr, br, bc, Ah[0]);


	unsigned long elapsed=0;
	int r;
	for (r = 0; r < reps; r++) {
#if USE_LUPD
		elapsed += lupd_main(diagl,t,br,bc,mt,nt,br,Ah,Ph,Sh);
#elif USE_LUPDH
		elapsed += lupdh_main(diagl, t, br, bc, h, mt, nt, Ah, Ph, Sh);
#elif USE_LUPDLL
		elapsed += lupdll_main(diagl,t,br,bc,mt,nt,br,Ah,Ph,Sh);
#elif USE_LUNAT
		elapsed += lunat_main(m, n, Ah, Ph);
#endif
  	}


	octave_matrix_def("U", m, mr, n, nr, br, bc, Ah[0]);
	lupd_check(check, m, mr, n, nr, br, bc, h);

	double dn = (double) n;
	double elapsedfl = (double) elapsed;
#if 0

  	//double dn   = (double) n;
	//printf( "n: %d time (secs.): %12.3f GFLOPS: %11.3f\n",
	//         n, t2/reps, (((2.0/3.0)*dn*dn*dn)/((t2/reps)*1.0e+9)) );
#else
  	FILE *statf=fopen("ompss-stats-0001","w");
	fprintf(statf,"time : %.2f\n",elapsedfl / (double) reps);
	fclose(statf);
#endif

	lu_shutdown();
	free(Ah);
	free(Sh);
	free(Ph);

	return 0;
}
