#include "lupd_check.h"
#include "lupd_utils.h"

#include <math.h>
#include <stdlib.h>

#if 0
// Perform equivalent to Octave's command:
// norm(triu(abs(M1)-abs(M2)),1)/norm(M1,1);
double __check ( double * M1, double * M2, int m, int n) {
	double *diffmat  = (double *) calloc( (m*n), sizeof(double) );
	if (diffmat == NULL) { perror("Error Allocating diffmat"); exit(1); }
	double *sumvec   = (double *) calloc( (n),   sizeof(double) );
	if (sumvec == NULL) { perror("Error Allocating sumvec"); exit(1); }
	double *sumvecM1 = (double *) calloc( (n),   sizeof(double) );
	if (sumvecM1 == NULL) { perror("Error Allocating sumvecM1"); exit(1); }


	int j;
	for (j=0; j<n; j++) {
		int r=min(j,m-1);
		int i;
		for (i=0; i<=r; i++) {
			double tmp = fabs(M1[j*m+i]);
			sumvecM1[j] += tmp;
			tmp -= fabs(M2[j*m+i]);
			diffmat[j*m+i] = tmp;
			sumvec[j] += fabs(tmp);
    		}
  	}

	double norm1=0.0, norm1M1=0.0;
	for (j=0; j<n; j++) {
		norm1= max(norm1,sumvec[j]);
		norm1M1 = max(norm1M1,sumvecM1[j]);
	}
	norm1 /= norm1M1;

	free(sumvec);
	free(sumvecM1);
	free(diffmat);

	return norm1;
}

int lupd_check(int check, int m, int mr, int n, int nr, int br, int bc, double **Ah, double *Aorig) {
	if( check > 0 ) {
		double *U = get_canonical_from_hierarchical( "U", m, mr, n, nr, br, bc, Ah[0] );
    
		int *IPIV = (int *) malloc( n * sizeof(int) );
		if (IPIV == NULL) { 
			perror("check: Error allocating IPIV"); 
			exit(1); 
		}

		printf( "calling dgetrf for residual checking..." );
		fflush(0);

		double norm_diff = 0.0;
		int IINFO;
		dgetrf_( &m, &n, Aorig, &m, IPIV, &IINFO );
		printf( "...done (IINFO: %d)\n", IINFO);
		if(IINFO < 0) {
			perror("\nError in dgeqrf\n");
			norm_diff = 1.0;
		}
		else {
			norm_diff = __check( Aorig, U, m, n);
			printf( "norm of difference: %22.16g\n", norm_diff);
		}

		free(Aorig);
		free(IPIV);
		free(U);

		int ret=(norm_diff > 2)?1:0;

		return ret;
	}

	return 0;
}
#endif


#if LUPD_CHECK == 1 
void lupd_check(int check, int m, int mr, int n, int nr, int br, int bc, int h) {
	if ( check ) {
		if ( LUPD_OCTAVE ) {
			//printf("%i %i %i %i %i %i h %i %i\n",m,mr,n,nr,br,bc,hr, hc);
			if ( n != m+bc ) {
				printf("warning: disabling error check (n!=m+bc, %i!=%i+%i)\n",n,m,bc);
			} else {
				fprintf(check_f, "B = A(:,%i+1:%i+%i);\n",m,m,bc);
				fprintf(check_f, "C = A(:,1:%i);\n",m);
				fprintf(check_f, "X = C \\ B;\n");
				fprintf(check_f, "U = triu(U);\n");
				fprintf(check_f, "U1 = U(:,1:%i);\n",m);
				fprintf(check_f, "U2 = U(:,%i+1:%i+%i);\n",m,m,bc);
				fprintf(check_f, "Xc = U1 \\ U2;\n");
	
				fprintf(check_f,"printf(\"%%.16f\\n\",norm(X-Xc,1)/norm(X,1));");
			}
		} else {
			fprintf(stderr, "warning: error check enabled but cannot generate Octave files\n");
		}
	}
}
#elif LUPD_CHECK == 2
void lupd_check(int check, int m, int mr, int n, int nr, int br, int bc, int h) {
	if ( check ) {
		if ( LUPD_OCTAVE ) {
			fprintf(check_f, "U1 = triu(U);\n");
#if USE_LUPDH
			fprintf(check_f, "U2 = lupdrect(A,%i,%i);\n",h,h);
#else
			fprintf(check_f, "U2 = lupdrect(A,%i,%i);\n",br,bc);
#endif
	
			fprintf(check_f,"printf(\"%%.16f\\n\",norm(U1-U2,1)/norm(U2,1));");
		} else {
			fprintf(stderr, "warning: error check enabled but cannot generate Octave files\n");
		}
	}
}
#endif


