#include "lupd_main.h"
#include "lupd_kernels.h"

#include <stdio.h>
#include <sys/time.h>


unsigned long lupd_main(int diagl, int t, int br, int bc, int mt, int nt, int ldim, double **Ah, int **Ph, double **Sh) {
	struct timeval start;
	gettimeofday( &start, NULL );

	int rk = 0;
	int k;
	for( k = 0; k < diagl; k++ ) {
		int row=rk / br;
		int rskip=rk % br;

		//printf("\niteration %i\n",k);

		// Factorize block on the diagonal block
		dgetrf_task( br, bc, rskip, Ah[ mt*k+row ], ldim, Ph[ mt*k+row ] );
		//printf("\tm: dgetrf (%i,%i) skip %i\n",row,k,rskip);


		int j;
		for( j = k+1; j < nt; j++ ) {
			dtrsm_dgemm_task( br, bc, rskip, Ah[ mt*k+row ], ldim, Ph[ mt*k+row ], Ah[ mt*j+row ]);
			//printf("\tm: dtrsm (%i,%i)->(%i,%i) skip %i\n",row,k,row,j,rskip);
		}

		int i;
		for( i = row+1; i < mt; i++ ) {
			double *test = Ah[mt*k+i];

			dgetrf_split_task( t, br, bc, rskip, Ah[ mt*k+row ], ldim, Ah[ mt*k+i ], Sh[ mt*k+i ], Ph[ mt*k+i ] );
			//printf("\tm: dgetrf_split (%i,%i) -> (%i,%i) skip %i\n",row,k,i,k,rskip);

			int j;
			for( j = k+1; j < nt; j++ ) {
				dtrsm_dgemm_split_task( t, br, bc, rskip, Ah[ mt*k+i ], ldim, Sh[ mt*k+i ], Ph[ mt*k+i ],
					Ah[ mt*j+row ], Ah[ mt*j+i ]);
				//printf("\tm: dtrsm_split (%i,%i) & (%i,%i) -> (%i,%i) skip %i\n",i,k,row,j,i,j,rskip);
			}
		}
		rk += bc;
	}

#pragma omp taskwait

	struct timeval stop;
	gettimeofday(&stop,NULL);

	unsigned long interm = stop.tv_usec - start.tv_usec;
	interm += (stop.tv_sec - start.tv_sec ) * 1000000;

	return interm;
}
