#include "lu_setup.h"


#if USE_LUPD || USE_LUPDLL


#include "lupd_check.h"
#include "lupd_utils.h"
#include <stdlib.h>
#include <time.h>


static double *A;
static double *S;
static int *P;

static char check_fname[]="lupdcheck_XXXXXX";
FILE *check_f=NULL;


int lu_setup(int check, int m, int mr, int n, int nr, int br, int bc, int t, int h, int tm, int tn, int mleft, int nleft, double ***Ah, double ***Sh, int ***Ph, double **Aorigp) {
	*Ah = (double **) malloc( tm * tn * sizeof(double *) );
	if(*Ah == NULL) return 1;
	*Sh = (double **) malloc( tm * tn * sizeof(double *) );
	if(*Sh == NULL) return 1;
	*Ph = (int **) malloc( tm * tn * sizeof(int *) );
	if(*Ph == NULL) return 1;

	double **lAh = *Ah;
	double **lSh = *Sh;
	int **lPh = *Ph;

	A = (double *) calloc( mr * nr, sizeof(double) );
	//printf("setup: A %x - %x\n", A, A + mr * nr - 1);
	// I assume rows > cols
	// for one block: bs * bs * ts / bs
	// for full matrix: bs * ts * mt * nt 
	S = (double *) malloc( t * nr * tm * sizeof(double) );
	P = (int *) malloc( tm * tn * bc * sizeof(int) );

	// Initialization of Ah, matrix with pointers to CONTIGIOUS blocks. 
	int j;
	for (j = 0; j < tn; j++) {
		int i;
		for (i = 0; i < tm; i++) {
			lAh[j*tm+i] = (double *) &A[j*bc*mr+i*br*bc];
      		}
  	}

	// Initialization of Sh 
	for (j = 0; j < tn; j++) {
		int i;
		for (i = 0; i < tm; i++) {
        		lSh[j*tm+i] = &S[j*tm*bc*t+i*bc*t];
		}
	}

	// Initialization of Ph 
	for (j = 0; j < tn; j++) {
		int i;
		for (i = 0; i < tm; i++) {
			lPh[j*tm+i] = &P[j*bc*tm+i*bc];
      		}
  	}

  
	// Initialization of A with random values
	srand48( ( long int ) time(NULL) );

	double *Ain = &A[0];
	for (j = 0; j < tn; j++) {
		int jend=j==tn-1;
		int i;
		for( i = 0; i < tm; i++) {
			int iend=i==tm-1;
			int jj; 
			for(jj=0 ; jj < bc; jj++) {
				int ii;
				for(ii=0 ; ii < br; ii++) {
  					int skipright = iend?ii>=br-mleft:0;
  					int skipbotm = jend?jj>=bc-nleft:0;
	  
					if(!skipright && !skipbotm) {
						double dran = drand48();
						if(dran == 0.0)  printf("generated 0\n");
						*Ain = dran;

						//printf("A(%i,%i)=%.4f\n",i*ts+ii+1,j*ts+jj+1,dran);
					}
  					Ain++;
				}
			//double *Ab = A[j * nt + i];
			//A[i] = drand48();
			}
    		}
  	}

	*Aorigp=NULL;
	if ( LUPD_OCTAVE ) {
		*Aorigp = canonical_from_blocked( "Aorig", m, mr, n, nr, br, bc, lAh[0] );
		mktemp(&(check_fname[0]));
		check_f = fopen(check_fname,"w");
	}

	return 0;
}


void lu_shutdown() {
	free(A);
	free(S);
	free(P);

	if ( check_f ) {
		fclose(check_f);
		printf("warning: wrote check file %s\n", check_fname);
	}
}


#endif // LUPD 
