#include "lupdh_main.h"

#include "lupd_kernels.h"
#include "lupdh_kernels.h"
#include "lupd_utils.h"
#include <stdio.h>
#include <sys/time.h>


unsigned long lupdh_main(int diagl, int t, int br, int bc, int h, int mt, int nt, double **Ah, int **Ph, double **Sh) {
	int btm = br / h;
	int btn = bc / h;
	int bdiagl = min(btm,btn);  

	//printf("%i steps with %ix%i superblocks of %ix%i blocks of %ix%i -> %ix%i\n", diagl, mt, nt, btm, btn, h, h, br, bc);  

	struct timeval start;
	gettimeofday( &start, NULL );

	
	int rk = 0;
	int k = 0;
	int d;
	while( d < diagl ) {
		//printf("\nstart iteration %i\n",k);
		double *F = Ah[ mt*2 + 0 ];
		double *Ft = &F[ br*0 + 0 ];
		//print_block_cm( stdout, "F", 4, 4, br, Ft);

		int row = d / btm;
		int rskip = d % btm;
		int col = d / btn;
		int cskip = d % btn;

		//printf("%i row %i %i col %i %i\n",d,row,rskip,col,cskip);

		//printf("m: %i dgetrf (%i,%i)\n",d,row,col);
		dgetrf_htask( rskip, cskip, btm, btn, h, t, Ah[ mt*col+row ], Ph[ mt*col+row ], Sh[ mt*col+row] );
		//void dgetrf_htask( int skip, int mb, int nb, int b, int t, double *A, int *P, double *S) {

		int j;
		for( j = col+1; j < nt; j++ ) {
			//printf("m: %i dtrsm_dgemm %i %i->%i\n", d, row, col, j);
			dtrsm_dgemm_htask( rskip, cskip, btm, btn, h, t, Ah[ mt*col+row ], Ph[ mt*col+row ], Sh[mt*col+row], Ah[ mt*j+row ]);
		}

		int i;
		for( i = row+1; i < mt; i++ ) {

			//printf("m: %i dgetrf_split %i  %i & %i\n", d, col, row, i);
			dgetrf_split_htask( rskip, cskip, btm, btn, h, t, Ah[ mt*col+row], Ah[mt*col+i], Ph[mt*col+i], Sh[mt*col+i]);

			int j;
			for( j = col+1; j < nt; j++ ) {
				//printf("m: %i dtrsm_dgemm_split (%i,%i) from l (%i,%i) t (%i,%i)\n", d, i, j, i, col, row, j);
				dtrsm_dgemm_split_htask( rskip, cskip, btm, btn, h, t, Sh[ mt*col+i ], Ah[ mt*col+i ], Ph[ mt*col+i ], Ah[ mt*j+row ], Ah[ mt*j+i ]);
			}
		}

		// i assume there are at least as many rows as columns in a block
		//d += (rskip==0?bdiagl:btm-rskip);
		d += min(btm-rskip,btn-cskip);
		++k;
	}

#pragma omp taskwait

	struct timeval stop;
	gettimeofday(&stop,NULL);

	unsigned long interm = stop.tv_usec - start.tv_usec;
	interm += (stop.tv_sec - start.tv_sec ) * 1000000;

	return interm;
}
