#include "lu_setup.h"


#if USE_LUPDH


#include "lupd_check.h"
#include "lupd_utils.h"
#include <stdlib.h>
#include <time.h>


static double *A;
static double *S;
static int *P;

static char check_fname[]="lupdhcheck_XXXXXX";
FILE *check_f=NULL;


int lu_setup(int check, int m, int mr, int n, int nr, int br, int bc, int t, int h, int tm, int tn, int mleft, int nleft, double ***Ah, double ***Sh, int ***Ph, double **Aorigp) {
	*Ah = (double **) malloc( tm * tn * sizeof(double *) );
	if(*Ah == NULL) return 1;
	*Sh = (double **) malloc( tm * tn * sizeof(double *) );
	if(*Sh == NULL) return 1;
	*Ph = (int **) malloc( tm * tn * sizeof(int *) );
	if(*Ph == NULL) return 1;

	double **lAh = *Ah;
	double **lSh = *Sh;
	int **lPh = *Ph;
	
	int bmt = br / h;
	int bnt = bc / h;

	// I assume rows > cols
	A = (double *) calloc( mr * nr, sizeof(double) );
	//printf("setup: A %x - %x\n", A, A + mr * nr - 1);
	// for one sub-block: t * h 
	// for one block: t * h * bmt * bnt 
	// for full matrix: t * h * bmt * bnt * mt * nt 
	S = (double *) malloc( t * h * bmt * bnt * tm * tn * sizeof(double) );
	//printf("S %x -> %x\n", S, &S[t * br * bdh * tm * tn - 1]); 
	// for one sub-block: h 
	// for one block: h * bmt * bnt 
	// for full matrix:  h * bmt * bnt * tm * tn
	P = (int *) malloc( h * bmt * bnt * tm * tn * sizeof(int) );

	// Initialization of Ah, matrix with pointers to CONTIGIOUS HIERARCHICAL blocks. 
	int j;
	for (j = 0; j < tn; j++) {
		int i;
		for (i = 0; i < tm; i++) {
			lAh[j*tm+i] = (double *) &A[j*bc*mr+i*br*bc];
      		}
  	}

	// Initialization of Sh 
	for (j = 0; j < tn; j++) {
		int i;
		for (i = 0; i < tm; i++) {
        		lSh[j*tm+i] = &S[j * tm*t*h*bmt*bnt + i * t*h*bmt*bnt];
		}
	}

	// Initialization of Ph 
	for (j = 0; j < tn; j++) {
		int i;
		for (i = 0; i < tm; i++) {
			lPh[j*tm+i] = &P[j * tm*h*bmt*bnt + i * h*bmt*bnt];
      		}
  	}

  
	// Initialization of A with random values
	srand48( ( long int ) time(NULL) );

	double *Ain = &A[0];
	for (j = 0; j < tn; j++) {
		int jend=j==tn-1;
		int i;
		for( i = 0; i < tm; i++) {
			int iend=i==tm-1;
			int jj; 
			for(jj=0 ; jj < bc; jj++) {
				int ii;
				for(ii=0 ; ii < br; ii++) {
  					int skipright = iend?ii>=br-mleft:0;
  					int skipbotm = jend?jj>=bc-nleft:0;
	  
					if(!skipright && !skipbotm) {
						double dran = drand48();
						if(dran == 0.0)  printf("generated 0\n");
						*Ain = dran;

						//printf("A(%i,%i)=%.4f\n",i*ts+ii+1,j*ts+jj+1,dran);
					}
  					Ain++;
				}
			//double *Ab = A[j * nt + i];
			//A[i] = drand48();
			}
    		}
  	}

	//print_matrix_blocked( stdout, "A", mr, mr, nr, nr, br, bc, lAh[0] );

	*Aorigp=NULL;
	if ( LUPD_OCTAVE ) {
		*Aorigp = canonical_from_blocked( "Aorig", m, mr, n, nr, br, bc, lAh[0] );
		mktemp(&(check_fname[0]));
		check_f = fopen(check_fname,"w");
	}

	return 0;
}


void lu_shutdown() {
	free(A);
	free(S);
	free(P);

	if ( check_f ) {
		fclose(check_f);
		printf("warning: wrote check file %s\n", check_fname);
	}
}


#endif // LUPDH
