#include "lupdh_kernels.h"
#include "lupd_kernels.h"
#include "lupd_utils.h"

#include <stdio.h>


#define Ab(i,j) ( &A[ cbs * j + i * b ] )
#define Pb(i,j) ( &P[ sbr * j + i * b ] )
#define Sb(i,j) ( &S[ ldimS * j + i * bt ] )


void dgetrf_htask( int rskip, int cskip, int mb, int nb, int b, int t, double *A, int *P, double *S) {
	int diagl = min(mb-rskip,nb-cskip);
	// super block dimensions
	int sbr = mb*b;
	// column block size
	int ldimS = sbr * t;
	int cbs = sbr * b;
	int bt = b * t;

	//printf("h: dgetrf diagl %i rskip %i\n", diagl, rskip);

	if ( diagl > 0) {
		double *Ad = Ab(rskip,cskip);
		int *Pd = Pb(rskip,cskip);
		int info;
		dgetrf_( &b, &b, Ad, &sbr, Pd, &info );
		if( info < 0 ) {
			printf( "dgetrf: error (%i)\n", info );
		}

		int j;
		for ( j = cskip+1 ; j < nb; j++ ) {
			double *Aj = Ab(rskip,j);
			dtrsm_dgemm_task(b, b, 0, Ad, sbr, Pd, Aj);
		}

		int i;
		for ( i=rskip+1 ; i < mb; i++ ) {
			double *Ai = Ab(i,cskip);
			int *Pi = Pb(i,cskip);
			double *Si = Sb(i,cskip);
			dgetrf_split_task( t, b, b, 0, Ad, sbr, Ai, Si, Pi);
			
			int j;
			for ( j = cskip+1; j<nb; j++ ) {
				double *Ajt = Ab(rskip,j);
				double *Ajb = Ab(i,j);
				dtrsm_dgemm_split_task( t, b, b, 0, Ai, sbr, Si, Pi, Ajt, Ajb ); 
			}
		}
	}


	int k;
	for ( k = 1; k < diagl; k++) {
		int r = rskip+k;
		int c = cskip+k;
		double *Ad = Ab(r,c);
		int *Pd = Pb(r,c);
		dgetrf_fac_task( b, b, Ad, sbr, Pd );

		int j;
		for ( j = c+1 ; j < nb; j++ ) {
			double *Aj = Ab(r,j);
			dtrsm_dgemm_task(b, b, 0, Ad, sbr, Pd, Aj);
		}

		int i;
		for ( i=r+1 ; i < mb; i++ ) {
			double *Ai = Ab(i,c);
			int *Pi = Pb(i,c);
			double *Si = Sb(i,c);
			dgetrf_split_task( t, b, b, 0, Ad, sbr, Ai, Si, Pi);
//			printf( "dgetrf_split\n");
			
			int j;
			for ( j = c+1; j<nb; j++ ) {
				double *Ajt = Ab(r,j);
				double *Ajb = Ab(i,j);
				dtrsm_dgemm_split_task( t, b, b, 0, Ai, sbr, Si, Pi, Ajt, Ajb ); 
//				printf( "dtrsm_dgemm_split\n");
			}
		}
	}
}


#undef Ab
#undef Pb
#undef Sb
