#include "lupdh_kernels.h"
#include "lupd_kernels.h"
#include "lupd_utils.h"

#include <stdio.h>


#define Ab(i,j) ( &A[ cbs * j + i * b ] )
#define Cb(i,j) ( &C[ cbs * j + i * b ] )
#define Pb(i,j) ( &P[ sbr * j + i * b ] )
#define Sb(i,j) ( &S[ ldimS * j + i * bt ] )


void dgetrf_split_htask( int rskip, int cskip, int mb, int nb, int b, int t, double *A, double *C, int *P, double *S ) {
	int diagl = min(mb-rskip,nb-cskip);
	// super block dimensions
	int sbr = mb*b;
	// column block size
	int cbs = sbr * b;
	int ldimS = sbr * t;
	int bt = b * t;


#if 1
	// i assume the superblocks are rectangular, so there will --probably-- 
	// be more than one dgetrf_split, hence it makes sense to execute these 
	// sequentially
	if ( diagl > 0 ) {
		double *Ak = Ab(rskip,cskip);

		double *Ai = Cb(0,cskip);
		int *Pi = Pb(0,cskip);
		double *Si = Sb(0,cskip);

		dgetrf_split( t, b, b, 0, Ak, sbr, Ai, Si, Pi);

		int j;
		for ( j=cskip+1 ; j<nb; j++) {
			double *Ajt = Ab(rskip,j);
			double *Ajb = Cb(0,j);
			dtrsm_dgemm_split_task( t, b, b, 0, Ai, sbr, Si, Pi, Ajt, Ajb ); 
		}

		int i;
		for ( i = 1 ; i < mb; i++ ) {
			double *Ai = Cb(i,cskip);
			int *Pi = Pb(i,cskip);
			double *Si = Sb(i,cskip);

			dgetrf_split_task( t, b, b, 0, Ak, sbr, Ai, Si, Pi);

			int j;
			for ( j=cskip+1 ; j<nb; j++) {
				double *Ajt = Ab(rskip,j);
				double *Ajb = Cb(i,j);
				dtrsm_dgemm_split_task( t, b, b, 0, Ai, sbr, Si, Pi, Ajt, Ajb ); 
			}
		}
	}
#endif


	int k;
	for ( k=1; k < diagl; k++ ) {
		int r = rskip+k;
		int c = cskip+k;
		double *Ak = Ab(r,c);

		int i;
		for ( i = 0 ; i < mb; i++ ) {
			double *Ai = Cb(i,c);
			int *Pi = Pb(i,c);
			double *Si = Sb(i,c);

			dgetrf_split_task( t, b, b, 0, Ak, sbr, Ai, Si, Pi);

			int j;
			for ( j=c+1 ; j<nb; j++) {
				double *Ajt = Ab(r,j);
				double *Ajb = Cb(i,j);
				dtrsm_dgemm_split_task( t, b, b, 0, Ai, sbr, Si, Pi, Ajt, Ajb ); 
			}
		}
	}
}


#undef Ab
#undef Cb
#undef Pb
#undef Sb
