#include "lupdh_kernels.h"
#include "lupd_kernels.h"
#include "lupd_utils.h"

#include <stdio.h>


#define Ab(i,j) ( &A[ cbs * j + i * b ] )
#define Pb(i,j) ( &P[ sbr * j + i * b ] )
#define Sb(i,j) ( &S[ ldimS * j + i * bt ] )
#define Cb(i,j) ( &C[ cbs * j + i * b ] )


void dtrsm_dgemm_htask( int rskip, int cskip, int mb, int nb, int b, int t, double *A, int *P, double *S, double *C ) { 
	int diagl = min(mb-rskip,nb-cskip);
	// super block dimensions
	int sbr = mb*b;
	// column block size
	int cbs = sbr * b;
	int ldimS = sbr * t;
	int bt = b * t;


#if 1
	// first iteration ...
	if ( diagl > 0 ) {
		double *Ak = Ab(rskip,cskip);
		int *Pk = Pb(rskip,cskip);

		// sequential code
		int j;
		for ( j=0; j<nb; j++) {
			double *Cj = Cb(rskip,j);
			dtrsm_dgemm(b, b, 0, Ak, sbr, Pk, Cj);
		}

		// parallel tasks
		int i;
		for ( i=rskip+1; i<mb; i++ ) {
			double *Si = Sb(i,cskip);
			int *Pi = Pb(i,cskip);
			double *Ai = Ab(i,cskip);

			int j;
			for ( j=0; j<nb; j++ ) {
				double *Cjt = Cb(rskip,j);
				double *Cjb = Cb(i,j);
				dtrsm_dgemm_split_task( t, b, b, 0, Ai, sbr, Si, Pi, Cjt, Cjb);
			}
		}
	}
#endif


	// ... rest of the iterations; because i m mixing sequential code and parallel tasks above, i 
	// have no choice but to use tasks exclusively
	int k;
	for ( k=0; k<diagl ; k++ ) {
		int r = rskip + k;
		int c = cskip + k;
		double *Ak = Ab(r,c);
		int *Pk = Pb(r,c);

		int j;
		for ( j=0; j<nb; j++) {
			double *Cj = Cb(r,j);
			dtrsm_dgemm_task(b, b, 0, Ak, sbr, Pk, Cj);
		}

		int i;
		for ( i=r+1; i<mb; i++ ) {
			double *Si = Sb(i,c);
			int *Pi = Pb(i,c);
			double *Ai = Ab(i,c);

			int j;
			for ( j=0; j<nb; j++ ) {
				double *Cjt = Cb(r,j);
				double *Cjb = Cb(i,j);
				dtrsm_dgemm_split_task( t, b, b, 0, Ai, sbr, Si, Pi, Cjt, Cjb);
			}
		}
	}
}


#undef Ab
#undef Pb
#undef Sb
#undef Cb
