#include "lupd_kernels.h"


/* Apply LUpiv factorization to block C:  C := inv( L ) * P * C.
   Arguments:
     L     in      m_C x m_C  Matrix with the gaussian transformations.
                              Defined by ( buff_L, ldim_L ).
     ipiv  in      m_C        Vector with pivots.
                              Defined by ( buff_ipiv ).
     C     in/out  m_C x n_C  Matrix to be udpated.
                              Defined by ( buff_C, ldim_C ).
*/


void dtrsm_dgemm_task( int m, int n, int skip, double *L, int ldim, int * IPIV, double *C ) {
	double done = 1.0;
	double mone = -1.0;
	int one = 1;

	L += skip;
	C += skip;
	int mC = m-skip;



	// Apply pivoting from IPIV to C
	dlaswp_( &n, C, &ldim, &one, &n, IPIV, &one );

	// Compute:  C := inv( L ) * C
  	dtrsm_( "Left", "Lower", "No transpose", "Unit", &n, &n, &done, L, &ldim, C, &ldim );

	double *Cl = C + n;
	mC -= n;
	L += n;

	dgemm_("No transpose", "No transpose", &mC, &n, &n, &mone, L, &ldim, C, &ldim, &done, Cl, &ldim);
}
