#include "lupd_kernels.h"
#include "NoFLA_Apply_td_ipiv_var1.h"

#include <stdio.h>


#define min( a, b ) ( (a) < (b) ? (a) : (b) )


/* Apply several block gaussian transformations defined by (Di,Pi,Si) to
   matrix [ F; G ] from left side.
     for i = 1, n_D/nb_alg
       [ F; G ] := apply_td_lu( [ I; Di ], Pi, Si, [ F; G ] ).
   Arguments:
     nb_alg                        Block size to be used in the factorization.
     D       in      m_G x n_D     Bottom part of matrix with gaussian trans.
                                   Defined by ( buff_D, ldim_D ).
     S       in      nb_alg x n_D  Matrix with triangular factors S.
                                   Defined by ( buff_S, ldim_S ).
     P       in      n_D           Vector with pivots.
                                   Defined by ( buff_P ).
     F       in/out  n_D x n_G     Top part of matrix to be updated.
                                   Defined by ( buff_F, ldim_F ).
     G       in/out  m_G x n_G     Bottom part of matrix to be updated.
                                   Defined by ( buff_G, ldim_G ).
*/
void dtrsm_dgemm_split_task( int t, int m, int n, int skip, double * buff_D, int ldim, double * buff_S, int * buff_P, double * buff_F, double * buff_G ) {
	double d_one = 1.0, d_m_one = -1.0;

	int ms = m - skip;
	buff_F += skip;

	int k;
	for( k = 0; k < n; k += t ) {
    		int b = min( t, n - k );

    		double *buff_D1 = &( buff_D[ ldim * k + 0 ] );  // D( 0, k )
		double *buff_F1 = &( buff_F[ ldim * 0 + k ] );  // F( k, 0 )
		double *buff_S1 = &( buff_S[ t * k + 0 ] );  // S( 0, k )
		int *buff_P1 = &( buff_P[ k ] );               // P( k )

		/* Apply pivoting from P1 to [ F; G ]. */
		NoFLA_Apply_td_ipiv_var1( b, n, m, buff_P1, buff_F1, ldim, buff_G, ldim, ms, k );

		/* F1 := inv( S1 ) * F1. */
		dtrsm_( "Left", "Lower", "No transpose", "Unit", &b, &n, &d_one, buff_S1, &t, buff_F1, &ldim );

		/* G := G - D1 * F1. */
		dgemm_( "No transpose", "No transpose", &m, &n, &b, &d_m_one, buff_D1, &ldim, buff_F1, &ldim, &d_one, buff_G, &ldim );
	}
}
