function A = lupdrect(A, br, bc)
# U overwrites A
# We don t compute L

if ( mod(br,bc) ) 
	printf("br must be a multiple of bc\n");
	exit(1);
endif
[m,n] = size(A);
mn = min(m,n);
diagl = ceil(mn / bc);
nt = ceil(n / bc);


rk = 0;
for k = 1:bc:mn
	row = floor(rk / br) * br + 1;
	rskip=mod(rk,br);

	cstart = k;
	cend = k + bc - 1;
	rstart = row;
	rend = row+br-1;

	Bf = A(rstart:rend,cstart:cend);
	[bfr,bfc] = size(Bf);
	Bc = Bf(1+rskip:bfr,:);
	[bcr,bcc] = size(Bc);
	
	# factorize the diagonal block
	[Lb, Ub, Pb] = lu(Bc);

	# update A with U
	A(rstart+rskip:rstart+rskip+bc-1,cstart:cend) = Ub;

	# Update the blocks on the right
  	Lbtop = Lb(1:bc,:);
	Linv = inv(Lbtop);
	Lbot = Lb(bc+1:bcr,:);
  	for j = k+bc:bc:n
		cs = j;
		ce = cs + bc - 1;

		# get the full panel, defined as C
		Cf = A(rstart:rend,cs:ce);
		C = Cf(1+rskip:br,:);
		[cr,cc] = size(C);
		# apply the pivoting 
		C = Pb * C;
		
		# compute the top Ct (lower triangular solve) and the bottom Cb (rank-bc update)
		Ct = Linv * C(1:bc,:);
		Cb = C(bc+1:cr,:) - Lbot * Ct;
		
		# update A
		A(rstart+rskip:rend,cs:ce) = vertcat(Ct,Cb);
  	end

	for i = row+br:br:m
		#printf("updating below\n");
		rs = i;
		re = i+br-1;

		# grab a block underneath
		Cf = A(rs:re,cstart:cend);

		D = vertcat(Ub,Cf);

		[Ld,Ud,Pd] = lu(D);
		[lr,lc] = size(Ld);
		A(rstart+rskip:rstart+rskip+bc-1,cstart:cend) = Ud;
		Ub = Ud;

		Ldinv = inv(Ld(1:bc,:));
		Ldb = Ld(bc+1:lr,:);

		#printf("updating below and right\n");
  		for j = k+bc:bc:n
			cs = j;
			ce = j+bc-1;
			
			Gf = A(rstart:rend,cs:ce);
			G = Gf(rskip+1:br,:)(1:bc,1:bc);

			H = A(rs:re,cs:ce);

			K = Pd * vertcat(G,H);

			G = Ldinv * K(1:bc,1:bc);
			H = K(bc+1:lr,:) - Ldb * G;

			A(rs:re,cs:ce) = H;
			A(rstart+rskip:rstart+rskip+bc-1,cs:ce) = G; 
		end 
	end

	rk += bc;
end

A = triu(A);
